@@ -1,7 +1,7 @@
 /* ------------------------------------------------------------ */
 /*
 HTTrack Website Copier, Offline Browser for Windows and Unix
-Copyright (C) 1998-2015 Xavier Roche and other contributors
+Copyright (C) 1998-2016 Xavier Roche and other contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -182,7 +182,7 @@ void help_wizard(httrackp * opt) {
   printf("\n");
   printf("Welcome to HTTrack Website Copier (Offline Browser) " HTTRACK_VERSION
          "%s\n", hts_get_version_info(opt));
-  printf("Copyright (C) 1998-2015 Xavier Roche and other contributors\n");
+  printf("Copyright (C) 1998-2016 Xavier Roche and other contributors\n");
 #ifdef _WIN32
   printf("Note: You are running the commandline version,\n");
   printf("run 'WinHTTrack.exe' to get the GUI version.\n");
@@ -299,7 +299,7 @@ void help_wizard(httrackp * opt) {
     // Options
     do {
       printf
-        ("\nYou can define additional options, such as recurse level (-r<number>), separed by blank spaces\n");
+        ("\nYou can define additional options, such as recurse level (-r<number>), separated by blank spaces\n");
       printf("To see the option list, type help\n");
       printf("Additional options (return=none) :");
       linput(stdin, str, 250);
@@ -795,7 +795,7 @@ void help(const char *app, int more) {
   sprintf(info, "HTTrack version " HTTRACK_VERSION "%s",
           hts_is_available());
   infomsg(info);
-  infomsg("Copyright (C) 1998-2015 Xavier Roche and other contributors");
+  infomsg("Copyright (C) 1998-2016 Xavier Roche and other contributors");
 #ifdef HTS_PLATFORM_NAME
   infomsg("[compiled: " HTS_PLATFORM_NAME "]");
 #endif
